# libraries
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns

from release_one.validation import format_tank, format_sales

# tank sticking for test formatting
dat = pd.read_csv('./data/tank/kt141_tank_history.csv')
dat = dat.loc[(dat.tank_id == 1)]
dat = dat.drop(dat.columns[4], axis=1)

# original data
plt.close()
sns.set()
sns.lineplot(y=dat.volume[:50], x=dat.read_time[:50])
plt.show()

# format tank sticking
tank_dat = dat.to_dict(orient="records")
tank_dat = format_tank(tank_dat)

# formatted data
plt.close()
sns.lineplot(y=tank_dat.y, x=tank_dat.ds)
plt.show()




# sales data for test formatting
dat = pd.read_csv('./data/sales/KT3116_sales.csv',
                  usecols=[1, 2, 3, 4, 5])

# original data
plt.close()
plt.plot(dat.sales)
plt.show()

# format data
sales_dat = dat.to_dict(orient="records")
sales_dat = format_sales(sales_dat)

# formatted data
plt.close()
plt.plot(sales_dat.y)
plt.show()



import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import seaborn as sns

from release_one.events import standard_events, apply_event_factor

generic_daily = pd.DataFrame({'ds': pd.date_range(start='2020-01-01',
                                                  end='2020-12-31', freq='1D'),
                              'yhat': np.repeat(100, 366)})

generic_hh = pd.DataFrame({'ds': pd.date_range(start='2020-12-24',
                                               end='2021-01-02', freq='30min'),
                           'yhat': np.repeat(100, 433)})


# long term forecast

sns.set()
sns.lineplot(x=generic_hh.ds, y=generic_hh.yhat)
plt.show()

test = apply_event_factor(generic_daily, standard_events)
test

sns.set()
sns.lineplot(x=test.ds, y=test.yhat)
plt.show()


# short term forecast

sns.set()
sns.lineplot(x=generic_hh.ds, y=generic_hh.yhat)
plt.show()

test2 = apply_event_factor(generic_hh, standard_events)
test2

sns.set()
sns.lineplot(x=test2.ds, y=test2.yhat)
plt.show()




import numpy as np
import pandas as pd
from datetime import datetime, timedelta
from fbprophet import Prophet

from tank_forecaster.decomp import generic_yearly_decomp, generic_weekly_decomp, generic_daily_decomp


def gen_future(df, periods=30, freq='1D'):
    start_0 = df.iloc[-1, 0]
    future = pd.date_range(start=start_0, freq=freq, periods=periods) + pd.Timedelta(freq)
    future = pd.DataFrame(future)
    future.rename(columns={0: 'ds'}, inplace=True)
    return future


def forecast_far(sales_history,
                 yearly_decomp,
                 weekly_decomp,
                 forecast_length=90,
                 daily_lift_est=0):

    # generate predictions
    predictions = pd.DataFrame(columns=['woy', 'dow', 'base', 'weekly', 'daily'])
    predictions['woy'] = np.repeat(range(1, 54), 7)  # 52.14 'week of year' -> 53
    predictions['dow'] = [0, 1, 2, 3, 4, 5, 6] * 53  # 7 'day of week' each week of year

    # no historical data
    if type(sales_history) != pd.core.frame.DataFrame:
        predictions['base'] = daily_lift_est
        weekly = np.repeat(predictions.base.iloc[1] * 7 * generic_yearly_decomp, 7)
        weekly.index = range(0, 371)
        predictions['weekly'] = weekly
        weekly_decomp_rep = pd.concat([generic_weekly_decomp] * 53, ignore_index=True)
        predictions['daily'] = (predictions.weekly * (1 / 7) * weekly_decomp_rep)
        data = {'ds': [pd.to_datetime(datetime.now()), pd.to_datetime(datetime.now())],
                'sales': [0, 0]}
        sales_history = pd.DataFrame(data=data)
        sales_history['date'] = sales_history['ds'].dt.date
    else:
        predictions['base'] = sales_history.y[-365:].mean()

    if len(yearly_decomp) <= 52:  # less than 1 year sales data, use generic yearly trends
        weekly = np.repeat(predictions.base[1] * 7 * generic_yearly_decomp, 7)
    else:  # more than 1 year sales data, use tank specific yearly trends
        weekly = np.repeat(predictions.base[1] * 7 * yearly_decomp, 7)

    weekly.index = range(0, 371)
    predictions['weekly'] = weekly
    weekly_decomp_rep = pd.concat([weekly_decomp] * 53, ignore_index=True)
    predictions['daily'] = (predictions.weekly * (1 / 7) * weekly_decomp_rep)

    # match predictions to future dataframe
    future = gen_future(sales_history, periods=forecast_length, freq='1D')
    future['dow'] = future.ds.dt.weekday
    future['woy'] = future.ds.dt.weekofyear
    output = pd.merge(future, predictions, left_on=['woy', 'dow'], right_on=['woy', 'dow'])
    output = output[['ds', 'daily']]
    output.rename(columns={'daily': 'yhat'}, inplace=True)

    # confidence interval
    output['lower'] = output['yhat'] - 2*output['yhat'].std()
    output['upper'] = output['yhat'] + 2*output['yhat'].std()

    # non-negative predictions
    for field in ["yhat", 'lower', 'upper']:
        output.loc[output[field] < 0, field] = 0

    return output


def forecast_near(validated_tank_history,
                  forecast_freq='30min',
                  forecast_length=144,
                  daily_lift_est=1000):

    # no historical data
    if type(validated_tank_history) != pd.core.frame.DataFrame:

        forecast = forecast_far(sales_history=[],
                                yearly_decomp=generic_yearly_decomp,
                                weekly_decomp=generic_weekly_decomp,
                                daily_lift_est=daily_lift_est,
                                forecast_length=3)

        now_hh = datetime.now() - (datetime.now() - datetime.min) % timedelta(minutes=30)
        fake_data = {'ds': [now_hh], 'sales': [0]}
        validated_tank_history = pd.DataFrame(data=fake_data)

        future = gen_future(validated_tank_history,
                            periods=forecast_length,
                            freq=forecast_freq)

        predictions = pd.DataFrame(columns=['ds', 'daily'])
        predictions['ds'] = future.ds
        predictions['daily'] = np.repeat(forecast.yhat[0], len(predictions))
        predictions['ts'] = predictions['ds'].dt.time

        predictions = pd.merge(left=predictions,
                               right=generic_daily_decomp,
                               how='left',
                               left_on='ts',
                               right_on='ts'
                               )

        predictions['hh'] = predictions.daily * (predictions.daily_multi * 1/48)
        output = predictions[['ds', 'hh']]
        output.rename(columns={'hh': 'yhat'}, inplace=True)

        return output

    # some historical data exists
    future = gen_future(validated_tank_history,
                        periods=forecast_length,
                        freq=forecast_freq)

    m = Prophet(changepoint_prior_scale=0.05,
                daily_seasonality=True,
                weekly_seasonality=True,
                yearly_seasonality=False)

    m.fit(validated_tank_history)

    forecast = m.predict(future)

    # format output
    output = forecast.loc[:, ['ds', 'yhat_lower', 'yhat_upper', 'yhat']]
    output.rename(columns={'yhat_lower': 'lower', 'yhat_upper': 'upper'}, inplace=True)

    # non-negative predictions
    for field in ["yhat", "lower", "upper"]:
        output.loc[output[field] < 0, field] = 0

    return output





# set up bar plot
holidays = decomp[2]
holiday_factors = holidays.iloc[[0, 146, 184, 246, 326, 360]]
holiday_names = pd.Series(['New Years', 'Memorial Day', '4 July',
                 'Labor Day', 'Thanks', 'Xmas'])
holiday_factors.reset_index(drop=True, inplace=True)


plt.close()
sns.set_palette(sns.color_palette("muted", 6))
fig = sns.barplot(y=holiday_factors, x=holiday_names).set_title(f'KT{store}: Fuel type {tank_type}')
plt.savefig(fname=f'../figures/gam_decomp/KT{store}{tank}_holiday_factor.png')
plt.show()


# plt.close()
# sns.set()
# fig, axs = plt.subplots(2)
# fig = sns.lineplot(y=generic_weekly_decomp, x=range(len(generic_weekly_decomp)), ax=axs[1])
# fig = sns.lineplot(y=generic_yearly_decomp, x=range(len(generic_yearly_decomp)), ax=axs[0]).set_title('Generic Seasonal Decomposition based on UNL 87')
# plt.savefig(fname=f'../figures/decompose/generic_seasonal_decomp.png')
# plt.show()



import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns

from release_one.validation import format_sales, format_tank, validate_tank
from release_one.forecaster import forecast_near


# load and format sales data
store = 144
tank = 2

dat = pd.read_csv(f'/Users/fisherankney/workspace/forecaster_local/data/sales/KT{store}{tank}_sales.csv',
                  usecols=range(1, 6))
tank_type = dat.tank_type.iloc[-1]
sales_dat = dat.to_dict(orient='records')
sales_dat = format_sales(sales_dat)


# load and format tank data
store = 144
tank = 1

dat = pd.read_csv(f'/Users/fisherankney/workspace/forecaster_local/data/tank/kt{store}_tank_history.csv')
dat = dat.loc[(dat.tank_id == tank)]
dat = dat.to_dict(orient="records")
tank_dat = format_tank(dat)


# test train split
train_full = tank_dat[:int(0.75*len(tank_dat))]
test = tank_dat[int(0.75*len(tank_dat)):]

# incomplete training data
train = train_full.iloc[-100:]
train

# if __name__ == '__main__':
train_val = validate_tank(tank_data=train, sales_data=sales_dat)
train_val

plt.close()
sns.set(rc={'figure.figsize':(11.7,8.27)})
fig = sns.lineplot(x=train_val.ds, y=train_val.actual, color='blue', alpha=0.9)
fig = sns.lineplot(x=train_val.ds, y=train_val.estimated, color='orange', alpha=0.9)
fig = sns.lineplot(x=train_val.ds, y=train_val.output, color='green', alpha=0.9)
fig = sns.lineplot(x=train_full.ds[-432:], y=train_full.y[-432:], color='black', alpha=0.4)
fig.legend(['actual (present)', 'estimated', 'used in model', 'actual (artificially missing)'])
plt.savefig(fname=f'./figures/forecast_near/missing_data_testing_10.png')
plt.show()

# short term forecast without validation
# forecast = forecast_near(train, forecast_length=144, output_format='df')
# forecast_v = forecast_near(train_val, forecast_length=144, output_format='df')


# visualize results
# plt.close()
# fig = sns.lineplot(x=forecast.ds, y=forecast.yhat, color='blue', alpha=0.7)
# fig = sns.lineplot(x=forecast_v.ds, y=forecast_v.yhat, color='green', alpha=0.7)
# fig = sns.lineplot(x=test.ds[:144], y=test.y[:144], color='black', alpha=0.7)
# fig.set_title(f'KT{store}: Fuel type {tank_type}')
# fig.legend(['no validation', 'validation', 'actual sales'])
# fig.set(ylabel='half-hour sales in gallons', xlabel='date')
# plt.savefig(fname=f'./figures/forecast_near/missing_data_testing_7.png')
# plt.show()




from release_one.events import standard_events, add_events, drop_events

# standard event list
standard_events

# adding new events
events = add_events(standard_events,
                    names=['july day', 'aug day'],
                    dates=['2020-07-03', '2020-08-01'],
                    factors=[2.5, 1.5])

events

# removing events
if __name__ == '__main__':
    events = drop_events(events,
                         names='july day',
                         dates='2020-07-03',
                         factors=2.5)

events


import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns

from release_one.validation import format_sales, format_tank, validate_tank
from release_one.forecaster import forecast_near, gen_future


# load and format sales data
store = 144
tank = 2

dat = pd.read_csv(f'/Users/fisherankney/workspace/forecaster_local/data/sales/KT{store}{tank}_sales.csv',
                  usecols=range(1, 6))
tank_type = dat.tank_type.iloc[-1]
sales_dat = dat.to_dict(orient='records')
sales_dat = format_sales(sales_dat)
sales_dat

# load and format tank data
store = 144
tank = 1

dat = pd.read_csv(f'/Users/fisherankney/workspace/forecaster_local/data/tank/old/kt{store}_tank_history.csv')
dat = dat.loc[(dat.tank_id == tank)]
dat = dat.to_dict(orient="records")
tank_dat = format_tank(dat)


# validate tank data
tank_dat = validate_tank(tank_dat, sales_dat)
tank_dat


# test train split
train = tank_dat[:int(0.74*len(tank_dat))]
test = tank_dat[int(0.74*len(tank_dat)):]

# short term forecast
forecast = forecast_near(train, forecast_length=144)


# visualize results
plt.close()
sns.set()
fig = sns.lineplot(x=test.ds[:144], y=test.y[:144], color='black', alpha=0.7)
fig = sns.lineplot(x=forecast.ds, y=forecast.yhat, color='blue', alpha=0.7)
fig.set_title('')
fig.legend(['Actual Sales', 'Forecasted Sales'])
fig.set(ylabel='Half-Hour Sales in Gallons', xlabel='Date')
# plt.savefig(fname=f'./figures/forecast_near/KT{store}{tank}_2.png')
plt.show()


